
//---------------------------------------------------------------------------------------------------------
// Utilities
//---------------------------------------------------------------------------------------------------------

// creating stringbundle to localize some strings
// var gBundle = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
// var strings = gBundle.createBundle("chrome://dragdropupload/locale/dragdropupload.properties");


/**
 * Given an INPUT field name extracts the prefix
 *
 * e.g. file-1 => file-
 */
function ddu_extractPrefix(name)
{
	if(!name)
		return "";
	var prefix = name;
	var namecount = 0;
	
	/// get the number from the ending of the prefix
	for(j = prefix.length-1; j >= 0; j--)
	{
		namecount = parseInt(prefix.substr(j));
		if(isNaN(namecount))
		{
			break;
		}
	}
	
	/// cases: all number, number in the end, no number
	if(j < 0)
	{
		prefix = "";							
	}
	else
	{		
		prefix = prefix.substring(0,j+1);
	}
	return prefix;
}

/**
 * Given an INPUT field name extracts the number
 *
 * e.g. file-1 => 1
 */
function ddu_extractNumber(name)
{
	if(!name)
		return NaN;
	var prefix = name;
	var namecount = NaN;
		
	/// get the number from the ending of the prefix
	for(j = prefix.length-1; j >= 0; j--)
	{
		namecount = parseInt(prefix.substr(j));
		if(isNaN(namecount))
		{
			break;
		}
	}
	
	/// cases: all number, number in the end, no number
	if(j < 0)
	{
	}
	else
	{		
		namecount = parseInt(prefix.substring(j+1));
	}
	return namecount;
}

function ddu_findParentForm(e,n)
{
	try
	{
		var s = e.nodeName
		e = e.parentNode;
		while(e && e.nodeName!= n)
		{
			s += " " + e.nodeName;
			e = e.parentNode;
		}
		if(!e)
		{
			//alert("DragDropUpload " + s);
		}
		return e;
	}
	catch(ee)
	{
		//alert("DragDropUpload ddu_findParentForm: " + ee);
	}
}

/**
 * Simulates the click of the specified node
 */
function ddu_simulateClick(node) 
{
    var event = node.ownerDocument.createEvent("MouseEvents");  
    event.initMouseEvent("click",
                         true, // can bubble
                         true, // cancellable
                         node.ownerDocument.defaultView,
                         1, // clicks
                         50, 50, // screen coordinates
                         50, 50, // client coordinates
                         false, false, false, false, // control/alt/shift/meta
                         0, // button,
                         node);  
    node.dispatchEvent(event);
}

/**
 * Simulates the click of the specified node
 */
function ddu_simulateOnChange(node) 
{
	try
	{
		var event = node.ownerDocument.createEvent("HTMLEvents");  
		event.initEvent("change",true,true);
		node.dispatchEvent(event);
	}
	catch(ex)
	{
//			alert("dd exception in simulateOnChange " + ex);
	}
}

/**
 * Trims the spaces of a string using regular expressions
 */  
function ddu_TrimString(sInString) {
  sInString = sInString.replace( /^\s+/g, "" );// strip leading
  return sInString.replace( /\s+$/g, "" );// strip trailing
}

function ddu_aAdd(r, a)
{
	for(var i = 0; i < a.length; ++i)
		r.push(a[i]);
}

/**
 * Recursive function that scans for INPUT fields in a page taking into account
 * frames and iframes that do not answer for getElementsByTagName
 */
function ddu_getElementsByTagNameDeep(page,name)
{	
	if (page == null) return [];
	//frames in the document are implemented in this way:
	//FRAMESET=>FRAME=>#document
	//HTMLFrameElement=>HTMLDocument
	
	var r = [];
	if(page == undefined)
		return r;
	var frames = page.getElementsByTagName("frame");
	for(var i = 0; i < frames.length; i++)
	{				
		var frame = frames[i];
		try
		{
			ddu_aAdd(r,ddu_getElementsByTagNameDeep(frame.contentDocument,name));
		}
		catch(e)
		{
			//alert("DragDropUpload getElementsByTagNameDeep: "+ e);
		}
	}	
	
	var frames = page.getElementsByTagName("iframe");
	for(var i = 0; i < frames.length; i++)
	{				
		var frame = frames[i];
		try
		{
			ddu_aAdd(r,ddu_getElementsByTagNameDeep(frame.contentDocument,name));
		}
		catch(e)
		{
			//alert("DragDropUpload getElementsByTagNameDeep: "+ e);
		}
	}	
	
	ddu_aAdd(r, page.getElementsByTagName(name));
	return r;
}


//---------------------------------------------------------------------------------------------------------
// Gmail Specific
//---------------------------------------------------------------------------------------------------------

/**
 * Recursive function that scans for INPUT fields in a page taking into account
 * frames and iframes that do not answer for getElementsByTagName
 */
function ddu_markAttachIt(page,name)
{	
	//frames in the document are implemented in this way:
	//FRAMESET=>FRAME=>#document
	//HTMLFrameElement=>HTMLDocument
	
	if(page == undefined)
		return;
		
	var frames = page.getElementsByTagName("frame");
	for(var i = 0; i < frames.length; i++)
	{				
		var frame = frames[i];
		ddu_markAttachIt(frame.contentDocument,name);
	}	
	
	var frames = page.getElementsByTagName("iframe");
	for(var i = 0; i < frames.length; i++)
	{				
		var frame = frames[i];
		ddu_markAttachIt(frame.contentDocument,name);			
	}	
	
	var q = page.getElementsByTagName(name)
	for(var k = 0; k < q.length;++k)
	{
		var x = q[k];
		var snt = x.innerHTML;
		var sn = typeof(snt) == "string" ? ddu_TrimString(snt) : "";
		if(sn == "Attach a file" )
		{
			if (!("_target" in x))
			{
				x.addEventListener("dragover", ddu_onover,false);
				x.addEventListener("dragdrop", ddu_ondrop,false);
				x.addEventListener("dragenter", ddu_showDragCursor, false);
				x._target = "SPAN";					/// special
			}
			throw(10);
		}
	}			
}

function ddu_makeGmailSpace(tform,n)
{
	var sib = tform.getElementsByTagName("INPUT");
	var base;
	try
	{
		if(sib.length == 0)
			base = tform;
		else
		{
			base = sib[0].parentNode.parentNode;
		}
		sib = base.getElementsByTagName("SPAN");
		for(var i = 0; i < sib.length; ++i)
		{
			var s = sib[i];
			var snt = s.innerHTML;
			//if(base != tform)
			//	alert(s + " " + snt);
			var sn = typeof(snt) == "string" ? ddu_TrimString(snt) : "";
			if(sn == "Attach another file")
			{			
				//for(var j = 0; j < n;++j)
				ddu_simulateClick(s);				
				n--;
				if(n == 0)
					return true;
				else
					return ddu_makeGmailSpace(tform,n);
			} 
		}
		return false;
	}
	catch(e)
	{
				//alert("DragDropUpload makeGmailSpace " + e);
		return false;
	}		
}

//---------------------------------------------------------------------------------------------------------
// Drag And Drop Handler
//---------------------------------------------------------------------------------------------------------

var ddu_dragDropHandler = 
{
	/// supports the x-moz-file shell object with the interface nsIFile
	/// supports the custom x2-moz-file for dropping from out sidebar
	/// supports the unicode as lines with file names
	getSupportedFlavours : function () {
		var flavours = new FlavourSet();
		flavours.appendFlavour("application/x-moz-file","nsIFile");	// real D&D
		flavours.appendFlavour("application/x2-moz-file"); // D&D from our script
		flavours.appendFlavour("text/unicode");
		return flavours;
	},
	onDragStart: function (evt , transferData, action){},
	onDragOver: function (evt,flavour,session) {},
	onDrop : function (evt, transferData, session) 
  {
    try
    {
	/*
		In the case of canHandleMultipleItems
		
		transferData is a TransferDataSet with the property datalist containing TransferData objects
		with the property datalist containing FlavourData that have (supports contentLength flavour _XferID data)
		
		TransferDataSet
			.dataList
				TransferData
					.dataList
						FlavourData
							supports
							contentLength
							_XferID
							flavour
								contentType
							data => gets the real data
				
		When it is off then transferData is the first item of the first flavour
	*/
	// prepare a variable that contains always a single item
	var td = this.canHandleMultipleItems ? transferData.first.first : transferData;
	var multi = (this.canHandleMultipleItems && transferData.dataList.length > 1) || (evt.target.tagName != "INPUT");
	
	switch(td.flavour.contentType)
	{
	case "application/x2-moz-file":
	case "text/unicode":
		{
			// check if there are multiple files ...
			if(multi)
			{
				// assume all of flavour moz-file
				var files = [];
				for(var i = 0; i < transferData.dataList.length; i++)
				{
					var td = transferData.dataList[i];
					for(var j = 0; j < td.dataList.length; j++)
					{
						var fd = td.dataList[j];
						if(fd.flavour.contentType == "application/x2-moz-file")
						{
							var s = fd.data + "";
							if(s.substr(0,8) == "file:///")
								s = s.substr(8);
							else if(s.substr(0,7) == "file://")
								s = s.substr(7);
							files.push(s);
						}
					}
				}
				this.applyFiles(evt, files);
			}
			else
			{
				var s = td.data + "";
				if(s.substr(0,8) == "file:///")
					s = s.substr(8);
				else if(s.substr(0,7) == "file://")
					s = s.substr(7);
				evt.target._target.value = s;	
				ddu_simulateOnChange(evt.target._target);							
			}
		}
		break;
	case "application/x-moz-file":
		{
			// check if there are multiple files ...
			if(multi)
			{
				// assume all of flavour moz-file
				var files = [];
				for(var i = 0; i < transferData.dataList.length; i++)
				{
					var td = transferData.dataList[i];
					for(var j = 0; j < td.dataList.length; j++)
					{
						var fd = td.dataList[j];
						if(fd.flavour.contentType == "application/x-moz-file")
						{
							files.push(fd.data.path);
						}
					}
				}
								
				this.applyFiles(evt, files);
			}
			else
			{
				evt.target._target.value = td.data.path; 
				ddu_simulateOnChange(evt.target._target);							
			}
		}
		break;
	}
	}
	catch(e)
	{
        //alert("DragDropUpload onDrop " + e);
	}
  },
  applyFiles: function(evt, files) 
  {
        try
        {
			var t = evt.target._target;
			var k = 0;
			var last = t;
			var tform;
			var d = 0;
			if(typeof(t) == "string")
			{
				tform = ddu_findParentForm(evt.target,"FORM");
				ddu_simulateClick(evt.target);
				d++;
				// look for inputs
			}
			else
			{
				tform = t.form;
				var prefix = ddu_extractPrefix(t.name);
				var namecount = ddu_extractNumber(t.name);
				//("First is " + prefix + " and "+ namecount);
				if(tform)
				{
					var sib = tform.getElementsByTagName("INPUT");
					for(var i = 0; i < sib.length && k < files.length; i++)
					{
						var f = sib[i];
							if(f.type == "file" && f.value == "")
							{
								var nn = ddu_extractNumber(f.name);
								if(!isNaN(nn) && (isNaN(namecount) || namecount < nn))
									namecount = nn;
								f.value = files[k++];
								ddu_simulateOnChange(f);								
								last = f;
							}
					}
				}
			}
			if(k < files.length)        
			{
				// OH OH
				if(!this.dynamicMode)
				{
					//alert("DragDropUpload applyFiles - Some files could not be dropped " + (files.length-k));
				}
				else
				{
					if(files.length-k-d == 0 || ddu_makeGmailSpace(tform,files.length-k-d))
					{
						var sib = tform.getElementsByTagName("INPUT");
						for(var i = 0; i < sib.length && k < files.length; i++)
						{
							var f = sib[i];
							if(f.type == "file" && f.value == "")
							{
								var nn = ddu_extractNumber(f.name);
								if(!isNaN(nn) && (isNaN(namecount) || namecount < nn))
									namecount = nn;
								f.value = files[k++];
								ddu_simulateOnChange(f);
								last = f;
							}
						}						
					}							
					else
					{
						/// it is actually a problem! => TODO disable dynamic																		
						for(; k < files.length; k++)
						{
							
							var x = tform.ownerDocument.createElement("INPUT");
							x.type = "file";
							x.size = t.size;
							x["onChange"] = t["onChange"];
							x["class"] = t["class"];
							//x["style"] = t["style"];
							if(isNaN(namecount))
								x.name = prefix;
							else
								x.name = prefix + (++namecount);
							x.value = files[k];							
							x.addEventListener("dragover", ddu_onover,false);
							x.addEventListener("dragdrop", ddu_ondrop,false);
							x.addEventListener("dragenter", ddu_showDragCursor, false);
							x._target = x;		
							ddu_simulateOnChange(x);							
							try
							{
								last.parentNode.insertBefore(x,last.nextSibling);				
							}
							catch(e)
							{
								tform.appendChild(x);
							}
							last = x;
						}					
					}
					if(k < files.length)
						alert("DragDropUpload applyFiles Some files could not be dropped " + (files.length-k));
				}
			}
		}
		catch(e)
		{
			//alert("DragDropUpload applyFiles Exception " + e);
		}  
  }
};

/**
 * Use our dragDropHandler
 */
function ddu_onover(event) 
{
	try {
		ddu_dragDropHandler.canHandleMultipleItems = true;
		ddu_dragDropHandler.dynamicMode = true;
		nsDragAndDrop.dragOver(event,ddu_dragDropHandler);
		//event.dataTransfer.dropEffect = "copy";
		event.returnValue = false;
	}
	catch(ex)
	{
	}
};

/**
 * show dragging cursor on over
 */
function ddu_showDragCursor(event) 
{
	event.returnValue = false;
	//event.dataTransfer.dropEffect = "copy";
}

/**
 * Use our dragDropHandler on Dropping
 */
function ddu_ondrop(event) 
{
	try 
	{
		nsDragAndDrop.drop(event,ddu_dragDropHandler);
	}
	catch(ex)
	{
	}
};

//---------------------------------------------------------------------------------------------------------
// Navigation Messages Observer
//---------------------------------------------------------------------------------------------------------

function ddu_NavObserver(oContentWindow)
{
	this.m_oContentWindow = oContentWindow;
	this.m_sWindowID = ''+parseInt(Math.random()*32767);
	this.m_sLastDataUrl = 'about:blank';	// The last url that we drove our display to.
}

ddu_NavObserver.prototype.observe =
	function (oSubject, sMessage, sContextUrl)
	{
		try {
			if (oSubject != this.m_oContentWindow) {
				// only pay attention to our client window.
				return;
			}
			var bReferrer = (this.m_oContentWindow.document.referrer)?true:false;
			if ((sMessage == 'EndDocumentLoad') 
				&& (sContextUrl != this.m_oContentWindow.location)) {
					// we were redirected...
				sContextUrl = '' + this.m_oContentWindow.location;
				bReferrer = true;
			}
			this.TrackContext(sContextUrl, bReferrer);
		} catch(ex) {
		}
	}

ddu_NavObserver.prototype.GetCDT =
	function (bReferrer)
	{
		var sCDT = '';
		sCDT += 't=' +(bReferrer?'1':'0');
		sCDT += '&pane=nswr6';
		sCDT += '&wid='+this.m_sWindowID;

		return encodeURIComponent(sCDT);
	}


ddu_NavObserver.prototype.TrackContext =
	function (sContextUrl, bReferrer)
	{
		try {
			// attach to the document change => get document loading complete => get list of input 
			// maybe attach to the document new element
			var win = this.oWindow;	
			if(win == null)
				return;
			var doc = this.oWindow.document;	
			if (doc.getElementById("content") != null) {
				var page = doc.getElementById("content").contentDocument;								
				try
				{
					ddu_markAttachIt(page,"span");
				}
				catch(e)
				{}
				var inputs = ddu_getElementsByTagNameDeep(page,"input");	
				var adj = 0;
				for(var i = 0; i < inputs.length; i++)
				{				
					var x = inputs[i];
					if(x.type == "file" && !("_target" in x))
					{
						x.addEventListener("dragover", ddu_onover,false);
						x.addEventListener("dragdrop", ddu_ondrop,false);
						x.addEventListener("dragenter", ddu_showDragCursor, false);
						x._target = x;
						adj++;
					}
				}
				// if(adj > 0) alert("Adjusted " + inputs.length + " fields");
			}
		}catch(ex) {			
			//alert("DragDropUpload exception in track  " + ex);
		}
	}

ddu_NavObserver.prototype.TranslateContext =
function (sUrl, bReferrer)
{
	if (!sUrl || ('string' != typeof(sUrl))
        || ('' == sUrl) || sUrl == 'about:blank') {
		return kUnknownReasonUrl;
	}

    // Strip off any query strings (Don't want to be too nosy).
	var nQueryPart = sUrl.indexOf('?');
    if (nQueryPart != 1) {
        sUrl = sUrl.slice(0, nQueryPart);
    }

	// We can only get related links data on HTTP URLs
	if (0 != sUrl.indexOf("http://")) {
		return kNoHTTPReasonUrl;
	}

	// ...and non-intranet sites(those that have a '.' in the domain)
	var sUrlSuffix = sUrl.substr(7);			// strip off "http://" prefix

	var nFirstSlash = sUrlSuffix.indexOf('/');
	var nFirstDot = sUrlSuffix.indexOf('.');

	if (-1 == nFirstDot)
		return kNoHTTPReasonUrl;

	if ((nFirstSlash < nFirstDot) && (-1 != nFirstSlash))
		return kNoHTTPReasonUrl;

	// url is http, non-intranet url: see if the domain is in their blocked list.

	var nPortOffset = sUrlSuffix.indexOf(":");
	var nDomainEnd = (((nPortOffset >=0) && (nPortOffset <= nFirstSlash))
                      ? nPortOffset : nFirstSlash);

	var sDomain = sUrlSuffix;
	if (-1 != nDomainEnd) {
		sDomain = sUrlSuffix.substr(0,nDomainEnd);
	}

	if (DomainInSkipList(sDomain)) {
		return kSkipDomainReasonUrl;
	} else {
		// ok! it is a good url!
		var sFinalUrl = kDataPrefixUrl;
		sFinalUrl += 'cdt='+this.GetCDT(bReferrer);
		sFinalUrl += '&url='+sUrl;
		return sFinalUrl;
	}
}

//---------------------------------------------------------------------------------------------------------
// Overlay Management
//---------------------------------------------------------------------------------------------------------

/**
 * This class scans the current Window and attaches the Navigation Messages Observer to the Document
 */
var ddu_UploadOverlay = {
	init: function(evt)
	{
		oContentWindow = window._content;
		
		if (oContentWindow) {
			var oObserverService = Components.classes["@mozilla.org/observer-service;1"].getService();
			oObserverService = oObserverService.QueryInterface(Components.interfaces.nsIObserverService);
	
			var oNavObserver = new ddu_NavObserver(oContentWindow);
			oNavObserver.TrackContext(''+oContentWindow.location);
			oNavObserver.oWindow = window;
	
			if (oObserverService && oNavObserver) {
				oObserverService.addObserver(oNavObserver, "StartDocumentLoad", false);
				oObserverService.addObserver(oNavObserver, "EndDocumentLoad", false);
				oObserverService.addObserver(oNavObserver, "FailDocumentLoad", false);
			} else {
				oNavObserver = null;
			}

			// Attach a timer to track the dynamic input fields			
			try
			{
				var s = ''+oContentWindow.location;
				oNavObserver.timeout = window.setInterval(
					function () { oNavObserver.TrackContext(s); },2000
				);
			}
			catch(e)
			{
				//alert("DragDropUpload init Exception " + e);
			}
		}	
	}
};


//---------------------------------------------------------------------------------------------------------
// Hooking to the Window and the current object
//---------------------------------------------------------------------------------------------------------

function ddu_initPitIntegration(evt) {
	ddu_UploadOverlay.init();
}

window.addEventListener("load", ddu_initPitIntegration, false);			// this works also for Mozilla
// addEventListener("load", ddu_initPitIntegration, false);				// required for Firebird
